@@ -1378,6 +1378,58 @@ emit_event_wrappers(struct wl_list *message_list, struct interface *interface)
 	}
 }
 
+static void
+emit_validator(struct interface *interface, struct enumeration *e)
+{
+	struct entry *entry;
+
+	printf("#ifndef %s_%s_ENUM_IS_VALID\n",
+	       interface->uppercase_name, e->uppercase_name);
+	printf("#define %s_%s_ENUM_IS_VALID\n",
+	       interface->uppercase_name, e->uppercase_name);
+
+	printf("/**\n"
+	       " * @ingroup iface_%s\n"
+	       " * Validate a %s %s value.\n"
+	       " *\n"
+	       " * @return true on success, false on error.\n"
+	       " * @ref %s_%s\n"
+	       " */\n"
+	       "static inline bool\n"
+	       "%s_%s_is_valid(uint32_t value, uint32_t version) {\n",
+	       interface->name, interface->name, e->name,
+	       interface->name, e->name,
+	       interface->name, e->name);
+
+	if (e->bitfield) {
+		printf("	uint32_t valid = 0;\n");
+		wl_list_for_each(entry, &e->entry_list, link) {
+			printf("	if (version >= %d)\n"
+			       "		valid |= %s_%s_%s;\n",
+			       entry->since,
+			       interface->uppercase_name, e->uppercase_name,
+			       entry->uppercase_name);
+		}
+		printf("	return (value & ~valid) == 0;\n");
+	} else {
+		printf("	switch (value) {\n");
+		wl_list_for_each(entry, &e->entry_list, link) {
+			printf("	case %s%s_%s_%s:\n"
+			       "		return version >= %d;\n",
+			       entry->value[0] == '-' ? "(uint32_t)" : "",
+			       interface->uppercase_name, e->uppercase_name,
+			       entry->uppercase_name, entry->since);
+		}
+		printf("	default:\n"
+		       "		return false;\n"
+		       "	}\n");
+	}
+	printf("}\n");
+
+	printf("#endif /* %s_%s_ENUM_IS_VALID */\n\n",
+	       interface->uppercase_name, e->uppercase_name);
+}
+
 static void
 emit_enumerations(struct interface *interface, bool with_validators)
 {
@@ -1439,35 +1491,11 @@ emit_enumerations(struct interface *interface, bool with_validators)
 
 		}
 
-		if (with_validators) {
-			printf("/**\n"
-			       " * @ingroup iface_%s\n"
-			       " * Validate a %s %s value.\n"
-			       " *\n"
-			       " * @return true on success, false on error.\n"
-			       " * @ref %s_%s\n"
-			       " */\n"
-			       "static inline bool\n"
-			       "%s_%s_is_valid(uint32_t value, uint32_t version) {\n"
-			       "	switch (value) {\n",
-			       interface->name, interface->name, e->name,
-			       interface->name, e->name,
-			       interface->name, e->name);
-			wl_list_for_each(entry, &e->entry_list, link) {
-				printf("	case %s%s_%s_%s:\n"
-				       "		return version >= %d;\n",
-				       entry->value[0] == '-' ? "(uint32_t)" : "",
-				       interface->uppercase_name, e->uppercase_name,
-				       entry->uppercase_name, entry->since);
-			}
-			printf("	default:\n"
-			       "		return false;\n"
-			       "	}\n"
-			       "}\n");
-		}
-
 		printf("#endif /* %s_%s_ENUM */\n\n",
 		       interface->uppercase_name, e->uppercase_name);
+
+		if (with_validators)
+			emit_validator(interface, e);
 	}
 }
 
